import * as Redux from "redux";
export declare type AutomataState<TState> = TState & {
    __sm_state?: string;
} & CanInvokeCapabilities;
export declare const ACTION_TYPE_PREFIX = "@@AUTOMATA";
export declare type ActionPayload = any;
export declare type CanInvokeFunction = <TPayload extends ActionPayload = undefined>(action: ActionDefinition<TPayload>) => boolean;
export interface CanInvokeCapabilities {
    canInvoke?: CanInvokeFunction;
}
export interface PayloadAction<TPayload extends ActionPayload = undefined> extends Redux.AnyAction {
    payload?: TPayload;
}
export interface AutomataAction<TPayload extends ActionPayload = undefined> extends PayloadAction<TPayload> {
    dispatch: Redux.Dispatch<AutomataAction<TPayload>>;
}
export interface ActionDefinition<TPayload extends ActionPayload = undefined> {
    actionType: string;
    (payload?: TPayload): PayloadAction<TPayload>;
}
export declare type TypedReducer<TState, TPayload extends ActionPayload = undefined> = (state: TState, arg: TPayload) => TState;
export interface StateDefinition<TState, TPayload extends ActionPayload = undefined> extends TypedReducer<TState, TPayload> {
    stateName: string;
}
export interface LocalStore<TState, TAction extends PayloadAction = Redux.AnyAction> extends Redux.Dispatch<TAction> {
    dispatch: Redux.Dispatch<TAction>;
    getState: () => TState;
}
export interface StateDefinition<TState, TPayload extends ActionPayload = undefined> extends TypedReducer<TState, TPayload> {
    stateName: string;
}
export declare type TransitionMethod<TState, TPayload extends ActionPayload = undefined> = (localStore: LocalStore<TState>, arg: TPayload) => void;
export interface StateMachineOptions<TState> {
    in<TPayload extends ActionPayload>(state: StateDefinition<TState, TPayload>): StateFluentOptions<TState>;
    inAny(): StateFluentOptions<TState>;
}
export interface StateFluentOptions<TState> {
    on<TPayload extends ActionPayload>(action: ActionDefinition<TPayload>): ActionFluentOptions<TState, TPayload>;
    or<TPayload extends ActionPayload>(state: StateDefinition<TState, TPayload>): StateFluentOptions<TState>;
}
export interface ActionFluentOptions<TState, TPayload extends ActionPayload = undefined> {
    goTo(state: StateDefinition<TState, TPayload>): StateFluentOptionsEx<TState>;
    execute(transition: TransitionMethod<TState, TPayload>): ActionFluentOptions<TState, TPayload>;
    noop(): StateFluentOptionsEx<TState>;
}
export interface StateFluentOptionsEx<TState> {
    in<TPayload extends ActionPayload>(state: StateDefinition<TState, TPayload>): StateFluentOptions<TState>;
    on<TPayload extends ActionPayload>(action: ActionDefinition<TPayload>): ActionFluentOptions<TState, TPayload>;
}
